@@ -2,6 +2,9 @@
 
 #include "gc.h"
 
+#include <glib/gi18n.h>
+#include <langinfo.h>
+#include <locale.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unicase.h>
@@ -10,6 +13,7 @@
 #include <uninorm.h>
 #include <unistr.h>
 #include "confusables.h"
+#include "scripts.h"
 
 #define PANGO_ENABLE_ENGINE 1
 #include <pango/pangofc-font.h>
@@ -76,7 +80,7 @@ struct GcCharacterIter
   size_t block_index;
   size_t block_count;
 
-  const uc_script_t *script;
+  const uc_script_t * const * scripts;
   uc_general_category_t category;
   const gchar * const * keywords;
 
@@ -113,33 +117,33 @@ gc_character_iter_next (GcCharacterIter *iter)
   while (TRUE)
     {
       if (uc == iter->blocks[iter->block_index].end)
-	{
-	  iter->block_index++;
-	  uc = UNINAME_INVALID;
-	}
+        {
+          iter->block_index++;
+          uc = UNINAME_INVALID;
+        }
 
       if (uc == UNINAME_INVALID)
-	{
-	  while (iter->block_index < iter->block_count
-		 && iter->blocks[iter->block_index].start
-		 == iter->blocks[iter->block_index].end)
-	    iter->block_index++;
-	  if (iter->block_index == iter->block_count)
-	    return FALSE;
-	  uc = iter->blocks[iter->block_index].start;
-	}
+        {
+          while (iter->block_index < iter->block_count
+                 && iter->blocks[iter->block_index].start
+                 == iter->blocks[iter->block_index].end)
+            iter->block_index++;
+          if (iter->block_index == iter->block_count)
+            return FALSE;
+          uc = iter->blocks[iter->block_index].start;
+        }
       else
-	uc++;
+        uc++;
 
       while (uc < iter->blocks[iter->block_index].end
-	     && !iter->filter (iter, uc))
-	uc++;
+             && !iter->filter (iter, uc))
+        uc++;
 
       if (uc < iter->blocks[iter->block_index].end)
-	{
-	  iter->uc = uc;
-	  return TRUE;
-	}
+        {
+          iter->uc = uc;
+          return TRUE;
+        }
     }
 
   return FALSE;
@@ -199,20 +203,32 @@ gc_character_iter_init_for_blocks (GcCharacterIter  *iter,
 }
 
 static gboolean
-filter_script (GcCharacterIter *iter, ucs4_t uc)
+filter_scripts (GcCharacterIter *iter, ucs4_t uc)
 {
-  return uc_is_print (uc) && uc_is_script (uc, iter->script);
+  const uc_script_t * const *scripts = iter->scripts;
+
+  if (!uc_is_print (uc))
+    return FALSE;
+
+  while (*scripts)
+    {
+      if (uc_is_script (uc, *scripts))
+        return TRUE;
+      scripts++;
+    }
+
+  return FALSE;
 }
 
 static void
-gc_character_iter_init_for_script (GcCharacterIter   *iter,
-                                   const uc_script_t *script)
+gc_character_iter_init_for_scripts (GcCharacterIter   *iter,
+                                    const uc_script_t * const * scripts)
 {
   gc_character_iter_init (iter);
   iter->blocks = all_blocks;
   iter->block_count = all_block_count;
-  iter->filter = filter_script;
-  iter->script = script;
+  iter->filter = filter_scripts;
+  iter->scripts = scripts;
 }
 
 static void
@@ -233,34 +249,34 @@ gc_enumerate_character_by_category (GcCharacterIter *iter,
 
     case GC_CATEGORY_ARROW:
       {
-	static uc_block_t arrow_blocks[3];
-	static gsize arrow_blocks_size = 0;
-	static gsize arrow_blocks_initialized = 0;
-	if (g_once_init_enter (&arrow_blocks_initialized))
-	  {
-	    const uc_block_t *block;
+        static uc_block_t arrow_blocks[3];
+        static gsize arrow_blocks_size = 0;
+        static gsize arrow_blocks_initialized = 0;
+        if (g_once_init_enter (&arrow_blocks_initialized))
+          {
+            const uc_block_t *block;
 
-	    /* 2190..21FF; Arrows */
-	    block = uc_block (0x2190);
-	    if (block)
-	      memcpy (&arrow_blocks[arrow_blocks_size++], block,
-		      sizeof (uc_block_t));
-	    /* 27F0..27FF; Supplemental Arrows-A */
-	    block = uc_block (0x27F0);
-	    if (block)
-	      memcpy (&arrow_blocks[arrow_blocks_size++], block,
-		      sizeof (uc_block_t));
-	    /* 2900..297F; Supplemental Arrows-B */
-	    block = uc_block (0x2900);
-	    if (block)
-	      memcpy (&arrow_blocks[arrow_blocks_size++], block,
-		      sizeof (uc_block_t));
-	    g_once_init_leave (&arrow_blocks_initialized, 1);
-	  }
-	gc_character_iter_init_for_blocks (iter,
-					   arrow_blocks,
-					   arrow_blocks_size);
-	return;
+            /* 2190..21FF; Arrows */
+            block = uc_block (0x2190);
+            if (block)
+              memcpy (&arrow_blocks[arrow_blocks_size++], block,
+                      sizeof (uc_block_t));
+            /* 27F0..27FF; Supplemental Arrows-A */
+            block = uc_block (0x27F0);
+            if (block)
+              memcpy (&arrow_blocks[arrow_blocks_size++], block,
+                      sizeof (uc_block_t));
+            /* 2900..297F; Supplemental Arrows-B */
+            block = uc_block (0x2900);
+            if (block)
+              memcpy (&arrow_blocks[arrow_blocks_size++], block,
+                      sizeof (uc_block_t));
+            g_once_init_leave (&arrow_blocks_initialized, 1);
+          }
+        gc_character_iter_init_for_blocks (iter,
+                                           arrow_blocks,
+                                           arrow_blocks_size);
+        return;
       }
 
     case GC_CATEGORY_BULLET:
@@ -272,49 +288,49 @@ gc_enumerate_character_by_category (GcCharacterIter *iter,
 
     case GC_CATEGORY_PICTURE:
       {
-	static uc_block_t picture_blocks[6];
-	static gsize picture_blocks_size = 0;
-	static gsize picture_blocks_initialized = 0;
-	if (g_once_init_enter (&picture_blocks_initialized))
-	  {
-	    const uc_block_t *block;
+        static uc_block_t picture_blocks[6];
+        static gsize picture_blocks_size = 0;
+        static gsize picture_blocks_initialized = 0;
+        if (g_once_init_enter (&picture_blocks_initialized))
+          {
+            const uc_block_t *block;
 
-	    /* 2600..26FF; Miscellaneous Symbols */
-	    block = uc_block (0x2600);
-	    if (block)
-	      memcpy (&picture_blocks[picture_blocks_size++], block,
-		      sizeof (uc_block_t));
-	    /* 2700..27BF; Dingbats */
-	    block = uc_block (0x2700);
-	    if (block)
-	      memcpy (&picture_blocks[picture_blocks_size++], block,
-		      sizeof (uc_block_t));
-	    /* 1F000..1F02F; Mahjong Tiles */
-	    block = uc_block (0x1F000);
-	    if (block)
-	      memcpy (&picture_blocks[picture_blocks_size++], block,
-		      sizeof (uc_block_t));
-	    /* 1F030..1F09F; Domino Tiles */
-	    block = uc_block (0x1F030);
-	    if (block)
-	      memcpy (&picture_blocks[picture_blocks_size++], block,
-		      sizeof (uc_block_t));
-	    /* 1F0A0..1F0FF; Playing Cards */
-	    block = uc_block (0x1F0A0);
-	    if (block)
-	      memcpy (&picture_blocks[picture_blocks_size++], block,
-		      sizeof (uc_block_t));
-	    /* 1F300..1F5FF; Miscellaneous Symbols and Pictographs */
-	    block = uc_block (0x1F300);
-	    if (block)
-	      memcpy (&picture_blocks[picture_blocks_size++], block,
-		      sizeof (uc_block_t));
-	    g_once_init_leave (&picture_blocks_initialized, 1);
-	  }
-	gc_character_iter_init_for_blocks (iter,
-					   picture_blocks,
-					   picture_blocks_size);
-	return;
+            /* 2600..26FF; Miscellaneous Symbols */
+            block = uc_block (0x2600);
+            if (block)
+              memcpy (&picture_blocks[picture_blocks_size++], block,
+                      sizeof (uc_block_t));
+            /* 2700..27BF; Dingbats */
+            block = uc_block (0x2700);
+            if (block)
+              memcpy (&picture_blocks[picture_blocks_size++], block,
+                      sizeof (uc_block_t));
+            /* 1F000..1F02F; Mahjong Tiles */
+            block = uc_block (0x1F000);
+            if (block)
+              memcpy (&picture_blocks[picture_blocks_size++], block,
+                      sizeof (uc_block_t));
+            /* 1F030..1F09F; Domino Tiles */
+            block = uc_block (0x1F030);
+            if (block)
+              memcpy (&picture_blocks[picture_blocks_size++], block,
+                      sizeof (uc_block_t));
+            /* 1F0A0..1F0FF; Playing Cards */
+            block = uc_block (0x1F0A0);
+            if (block)
+              memcpy (&picture_blocks[picture_blocks_size++], block,
+                      sizeof (uc_block_t));
+            /* 1F300..1F5FF; Miscellaneous Symbols and Pictographs */
+            block = uc_block (0x1F300);
+            if (block)
+              memcpy (&picture_blocks[picture_blocks_size++], block,
+                      sizeof (uc_block_t));
+            g_once_init_leave (&picture_blocks_initialized, 1);
+          }
+        gc_character_iter_init_for_blocks (iter,
+                                           picture_blocks,
+                                           picture_blocks_size);
+        return;
       }
       break;
 
@@ -327,30 +343,35 @@ gc_enumerate_character_by_category (GcCharacterIter *iter,
       return;
 
     case GC_CATEGORY_LATIN:
-      gc_character_iter_init_for_script (iter, uc_script ('A'));
-      return;
+      {
+        static const uc_script_t *latin_scripts[2];
+        latin_scripts[0] = uc_script ('A');
+        latin_scripts[1] = NULL;
+        gc_character_iter_init_for_scripts (iter, latin_scripts);
+        return;
+      }
 
     case GC_CATEGORY_EMOTICON:
       {
-	static uc_block_t emoticon_blocks[1];
-	static gsize emoticon_blocks_size = 0;
-	static gsize emoticon_blocks_initialized = 0;
-	if (g_once_init_enter (&emoticon_blocks_initialized))
-	  {
-	    const uc_block_t *block;
+        static uc_block_t emoticon_blocks[1];
+        static gsize emoticon_blocks_size = 0;
+        static gsize emoticon_blocks_initialized = 0;
+        if (g_once_init_enter (&emoticon_blocks_initialized))
+          {
+            const uc_block_t *block;
 
-	    /* 1F600..1F64F; Emoticons */
-	    block = uc_block (0x1F600);
-	    if (block)
-	      memcpy (&emoticon_blocks[emoticon_blocks_size++], block,
-		      sizeof (uc_block_t));
-	    g_once_init_leave (&emoticon_blocks_initialized, 1);
-	  }
-	gc_character_iter_init_for_blocks (iter,
-					   emoticon_blocks,
-					   emoticon_blocks_size);
-	iter->filter = filter_is_print;
-	return;
+            /* 1F600..1F64F; Emoticons */
+            block = uc_block (0x1F600);
+            if (block)
+              memcpy (&emoticon_blocks[emoticon_blocks_size++], block,
+                      sizeof (uc_block_t));
+            g_once_init_leave (&emoticon_blocks_initialized, 1);
+          }
+        gc_character_iter_init_for_blocks (iter,
+                                           emoticon_blocks,
+                                           emoticon_blocks_size);
+        iter->filter = filter_is_print;
+        return;
       }
     }
 
@@ -377,13 +398,13 @@ filter_keywords (GcCharacterIter *iter, ucs4_t uc)
       /* Match against the character itself.  */
       u32_to_u8 (&uc, 1, utf8, &utf8_length);
       if (utf8_length == length && memcmp (*keywords, utf8, utf8_length) == 0)
-	return TRUE;
+        return TRUE;
 
       /* Match against the hexadecimal code point.  */
       if (length <= 6
-	  && strspn (*keywords, "0123456789abcdefABCDEF") == length
-	  && strtoul (*keywords, NULL, 16) == uc)
-	return TRUE;
+          && strspn (*keywords, "0123456789abcdefABCDEF") == length
+          && strtoul (*keywords, NULL, 16) == uc)
+        return TRUE;
     }
 
   /* Match against the name.  */
@@ -422,7 +443,7 @@ gc_character_name (gunichar uc)
 }
 
 G_DEFINE_BOXED_TYPE (GcSearchResult, gc_search_result,
-		     g_array_ref, g_array_unref);
+                     g_array_ref, g_array_unref);
 
 gunichar
 gc_search_result_get (GcSearchResult *result, gint index)
@@ -437,6 +458,7 @@ struct SearchData
 {
   GcCategory category;
   gchar **keywords;
+  const uc_script_t **scripts;
   gunichar uc;
   gint max_matches;
 };
@@ -446,6 +468,8 @@ search_data_free (struct SearchData *data)
 {
   if (data->keywords)
     g_strfreev (data->keywords);
+  if (data->scripts)
+    g_free (data->scripts);
   g_slice_free (struct SearchData, data);
 }
 
@@ -465,11 +489,11 @@ gc_search_by_category_thread (GTask        *task,
   result = g_array_new (FALSE, FALSE, sizeof (gunichar));
   gc_enumerate_character_by_category (&iter, data->category);
   while (!g_cancellable_is_cancelled (cancellable)
-	 && gc_character_iter_next (&iter))
+         && gc_character_iter_next (&iter))
     {
       gunichar uc = gc_character_iter_get (&iter);
       if (data->max_matches < 0 || result->len < data->max_matches)
-	g_array_append_val (result, uc);
+        g_array_append_val (result, uc);
     }
 
   g_task_return_pointer (task, result, (GDestroyNotify) g_array_unref);
@@ -499,7 +523,7 @@ gc_search_by_category (GcCategory          category,
   data->category = category;
   data->max_matches = max_matches;
   g_task_set_task_data (task, data,
-			(GDestroyNotify) search_data_free);
+                        (GDestroyNotify) search_data_free);
   g_task_run_in_thread (task, gc_search_by_category_thread);
 }
 
@@ -520,11 +544,11 @@ gc_search_by_keywords_thread (GTask        *task,
   result = g_array_new (FALSE, FALSE, sizeof (gunichar));
   gc_enumerate_character_by_keywords (&iter, keywords);
   while (!g_cancellable_is_cancelled (cancellable)
-	 && gc_character_iter_next (&iter))
+         && gc_character_iter_next (&iter))
     {
       gunichar uc = gc_character_iter_get (&iter);
       if (data->max_matches < 0 || result->len < data->max_matches)
-	g_array_append_val (result, uc);
+        g_array_append_val (result, uc);
     }
 
   g_task_return_pointer (task, result, (GDestroyNotify) g_array_unref);
@@ -540,10 +564,10 @@ gc_search_by_keywords_thread (GTask        *task,
  */
 void
 gc_search_by_keywords (const gchar * const * keywords,
-		       gint                  max_matches,
-		       GCancellable         *cancellable,
-		       GAsyncReadyCallback   callback,
-		       gpointer              user_data)
+                       gint                  max_matches,
+                       GCancellable         *cancellable,
+                       GAsyncReadyCallback   callback,
+                       gpointer              user_data)
 {
   GTask *task;
   struct SearchData *data;
@@ -554,13 +578,71 @@ gc_search_by_keywords (const gchar * const * keywords,
   data->keywords = g_strdupv ((gchar **) keywords);
   data->max_matches = max_matches;
   g_task_set_task_data (task, data,
-			(GDestroyNotify) search_data_free);
+                        (GDestroyNotify) search_data_free);
   g_task_run_in_thread (task, gc_search_by_keywords_thread);
 }
 
+static void
+gc_search_by_scripts_thread (GTask        *task,
+                             gpointer      source_object,
+                             gpointer      task_data,
+                             GCancellable *cancellable)
+{
+  GcCharacterIter iter;
+  GArray *result;
+  struct SearchData *data = task_data;
+
+  if (!all_blocks)
+    uc_all_blocks (&all_blocks, &all_block_count);
+
+  result = g_array_new (FALSE, FALSE, sizeof (gunichar));
+  gc_character_iter_init_for_scripts (&iter,
+                                      (const uc_script_t * const *) data->scripts);
+  while (!g_cancellable_is_cancelled (cancellable)
+         && gc_character_iter_next (&iter))
+    {
+      gunichar uc = gc_character_iter_get (&iter);
+      g_array_append_val (result, uc);
+    }
+
+  g_task_return_pointer (task, result, (GDestroyNotify) g_array_unref);
+}
+
+/**
+ * gc_search_by_scripts:
+ * @scripts: (array zero-terminated=1) (element-type utf8): an array of scripts
+ * @max_matches: the maximum number of results.
+ * @cancellable: a #GCancellable.
+ * @callback: a #GAsyncReadyCallback.
+ * @user_data: a user data passed to @callback.
+ */
+void
+gc_search_by_scripts (const gchar * const * scripts,
+                      gint                  max_matches,
+                      GCancellable         *cancellable,
+                      GAsyncReadyCallback   callback,
+                      gpointer              user_data)
+{
+  GTask *task;
+  struct SearchData *data;
+  guint length, i;
+
+  task = g_task_new (NULL, cancellable, callback, user_data);
+
+  data = g_slice_new0 (struct SearchData);
+  length = g_strv_length ((gchar **) scripts);
+  data->scripts = g_malloc0_n (length + 1, sizeof (uc_script_t *));
+  for (i = 0; i < length; i++)
+    data->scripts[i] = uc_script_byname (scripts[i]);
+  data->max_matches = max_matches;
+  g_task_set_task_data (task, data,
+                        (GDestroyNotify) search_data_free);
+  g_task_run_in_thread (task, gc_search_by_scripts_thread);
+}
+
 static int
 confusable_character_class_compare (const void *a,
-				    const void *b)
+                                    const void *b)
 {
   const struct ConfusableCharacterClass *ac = a, *bc = b;
   return ac->uc == bc->uc ? 0 : (ac->uc < bc->uc ? -1 : 1);
@@ -575,21 +657,21 @@ gc_add_confusables (struct SearchData *data,
 
   key.uc = data->uc;
   res = bsearch (&key, confusable_character_classes,
-		 G_N_ELEMENTS (confusable_character_classes),
-		 sizeof (*confusable_character_classes),
-		 confusable_character_class_compare);
+                 G_N_ELEMENTS (confusable_character_classes),
+                 sizeof (*confusable_character_classes),
+                 confusable_character_class_compare);
   if (res)
     {
       const struct ConfusableClass *klass = &confusable_classes[res->index];
       uint16_t i;
 
       for (i = 0; i < klass->length
-	     && !g_cancellable_is_cancelled (cancellable); i++)
-	{
-	  gunichar uc = confusable_characters[klass->offset + i];
-	  if (data->max_matches < 0 || result->len < data->max_matches)
-	    g_array_append_val (result, uc);
-	}
+             && !g_cancellable_is_cancelled (cancellable); i++)
+        {
+          gunichar uc = confusable_characters[klass->offset + i];
+          if (data->max_matches < 0 || result->len < data->max_matches)
+            g_array_append_val (result, uc);
+        }
     }
 }
 
@@ -613,23 +695,23 @@ remove_duplicates (GArray *array)
 
       start = &g_array_index (array, gunichar, i);
       for (j = i + 1; j < array->len; j++)
-	{
-	  gunichar *stop;
+        {
+          gunichar *stop;
 
-	  stop = &g_array_index (array, gunichar, j);
-	  if (*start != *stop)
-	    break;
-	}
+          stop = &g_array_index (array, gunichar, j);
+          if (*start != *stop)
+            break;
+        }
       if (j != i + 1)
-	g_array_remove_range (array, i + 1, j - (i + 1));
+        g_array_remove_range (array, i + 1, j - (i + 1));
     }
 }
 
 static void
 add_composited (GArray *result,
-		ucs4_t uc,
-		ucs4_t *block_characters,
-		size_t n_block_characters)
+                ucs4_t uc,
+                ucs4_t *block_characters,
+                size_t n_block_characters)
 {
   ucs4_t decomposition[UC_DECOMPOSITION_MAX_LENGTH];
   int decomposition_length;
@@ -651,19 +733,19 @@ add_composited (GArray *result,
 
       block = uc_block (block_characters[i]);
       for (uc = block->start; uc < block->end; uc++)
-	{
-	  decomposition_length = uc_canonical_decomposition (uc, decomposition);
-	  if (decomposition_length > 0 && decomposition[0] == base)
-	    g_array_append_val (result, uc);
-	}
+        {
+          decomposition_length = uc_canonical_decomposition (uc, decomposition);
+          if (decomposition_length > 0 && decomposition[0] == base)
+            g_array_append_val (result, uc);
+        }
     }
 }
 
 static void
 gc_search_related_thread (GTask        *task,
-			  gpointer      source_object,
-			  gpointer      task_data,
-			  GCancellable *cancellable)
+                          gpointer      source_object,
+                          gpointer      task_data,
+                          GCancellable *cancellable)
 {
   GArray *result;
   struct SearchData *data = task_data;
@@ -694,26 +776,26 @@ gc_search_related_thread (GTask        *task,
 
       script = uc_script (data->uc);
       if (script)
-	{
-	  if (strcmp (script->name, "Latin") == 0)
-	    {
-	      ucs4_t block_starters[4] = { 0x0000, 0x0080, 0x0100, 0x0180 };
-	      add_composited (result, data->uc,
-			      block_starters, G_N_ELEMENTS (block_starters));
-	    }
-	  else if (strcmp (script->name, "Hiragana") == 0)
-	    {
-	      ucs4_t block_starters[1] = { 0x3040 };
-	      add_composited (result, data->uc,
-			      block_starters, G_N_ELEMENTS (block_starters));
-	    }
-	  else if (strcmp (script->name, "Katakana") == 0)
-	    {
-	      ucs4_t block_starters[2] = { 0x30A0, 0x31F0 };
-	      add_composited (result, data->uc,
-			      block_starters, G_N_ELEMENTS (block_starters));
-	    }
-	}
+        {
+          if (strcmp (script->name, "Latin") == 0)
+            {
+              ucs4_t block_starters[4] = { 0x0000, 0x0080, 0x0100, 0x0180 };
+              add_composited (result, data->uc,
+                              block_starters, G_N_ELEMENTS (block_starters));
+            }
+          else if (strcmp (script->name, "Hiragana") == 0)
+            {
+              ucs4_t block_starters[1] = { 0x3040 };
+              add_composited (result, data->uc,
+                              block_starters, G_N_ELEMENTS (block_starters));
+            }
+          else if (strcmp (script->name, "Katakana") == 0)
+            {
+              ucs4_t block_starters[2] = { 0x30A0, 0x31F0 };
+              add_composited (result, data->uc,
+                              block_starters, G_N_ELEMENTS (block_starters));
+            }
+        }
     }
 
   gc_add_confusables (data, result, cancellable);
@@ -726,10 +808,10 @@ gc_search_related_thread (GTask        *task,
 
       puc = &g_array_index (result, gunichar, i);
       if (*puc == data->uc)
-	{
-	  g_array_remove_index (result, i);
-	  break;
-	}
+        {
+          g_array_remove_index (result, i);
+          break;
+        }
     }
 
   g_task_return_pointer (task, result, (GDestroyNotify) g_array_unref);
@@ -759,7 +841,7 @@ gc_search_related (gunichar            uc,
   data->uc = uc;
   data->max_matches = max_matches;
   g_task_set_task_data (task, data,
-			(GDestroyNotify) search_data_free);
+                        (GDestroyNotify) search_data_free);
   g_task_run_in_thread (task, gc_search_related_thread);
 }
 
@@ -772,7 +854,7 @@ gc_search_related (gunichar            uc,
  */
 GcSearchResult *
 gc_search_finish (GAsyncResult *result,
-                            GError      **error)
+                  GError      **error)
 {
   g_return_val_if_fail (g_task_is_valid (result, NULL), NULL);
 
@@ -839,3 +921,76 @@ gc_pango_context_font_has_glyph (PangoContext *context,
 
   return retval == 0;
 }
+
+/**
+ * gc_get_current_language:
+ *
+ * Returns: (transfer full): an ISO639 two-letter language code
+ */
+gchar *
+gc_get_current_language (void)
+{
+  const gchar *locale = setlocale (LC_MESSAGES, NULL);
+  size_t length;
+
+  if (!locale || !*locale)
+    return NULL;
+
+  length = strcspn (locale, "_.@");
+
+  return g_strndup (locale, length);
+}
+
+static int
+language_scripts_compare (const void *a,
+                          const void *b)
+{
+  const struct LanguageScripts *al = a, *bl = b;
+  return strcmp (al->language, bl->language);
+}
+
+static int
+language_scripts_compare_ignore_territory (const void *a,
+                                           const void *b)
+{
+  const struct LanguageScripts *al = a, *bl = b;
+  int an, bn;
+  gchar *p;
+
+  p = strchr (al->language, '_');
+  an = p ? p - al->language : strlen (al->language);
+  p = strchr (bl->language, '_');
+  bn = p ? p - bl->language : strlen (bl->language);
+
+  if (an == bn)
+    return strncmp (al->language, bl->language, an);
+
+  return language_scripts_compare (a, b);
+}
+
+/**
+ * gc_get_scripts_for_language:
+ * @language: a language name
+ *
+ * Returns: (transfer none) (array zero-terminated=1) (element-type utf8): a list of script names.
+ */
+const gchar * const *
+gc_get_scripts_for_language (const gchar *language)
+{
+  struct LanguageScripts key, *res;
+
+  key.language = language;
+  res = bsearch (&key, language_scripts,
+                 G_N_ELEMENTS (language_scripts),
+                 sizeof (*language_scripts),
+                 language_scripts_compare);
+  if (!res)
+    res = bsearch (&key, language_scripts,
+                   G_N_ELEMENTS (language_scripts),
+                   sizeof (*language_scripts),
+                   language_scripts_compare_ignore_territory);
+  if (res)
+    return res->scripts;
+
+  return NULL;
+}
